function [By,Bx,omega]=rfvardraws_dummyobs(ydata,lags,xdata,ndraws, hyperparameters, prior_only)
% Code based on rfvar routine by Chris Sims written and Schorfheide et al. for dummy observation generation
% 
% written for Amir-Ahmadi and Drautzburg (2019)
% Last modifed 11/25/19 
% 
% Wishart and Gaussian draws for a reduced-form VAR
% ydata:   dependent variable data matrix
% xdata:   exogenous variable data matrix
% lags:    number of lags
% lambda:  the weight on the dummy observation favoring a unit root or stationary 
%          models with small initial transient.  
% mu:      the weight on dummy observations that favor a separate unit root for 
%          each variable.
% lambda3: prior for diagonal covariance matrix
%  A reasonable starting point is lambda=mu=1.  lambda=mu=0 produces OLS.

if ~exist('prior_only', 'var')
    prior_only = 0;
end
if ~exist('hyperparameters', 'var')

    hyperparameters.tau     =   .5;             % 0.1 overall tightness */
    hyperparameters.d       =   4;               % scaling down the variance for the coefficients of a distant lag */
    hyperparameters.w       =   1;               % number of observations used for obtaining the prior for the covariance matrix of error terms*/
    hyperparameters.lambda  =   0.5;               % 2.3 tuning parameter for coefficients for constant*/
    hyperparameters.mu      =   0.5;               % 12 tuning parameter for the covariance between coefficients*/
end
if prior_only==1 && hyperparameters.tau*hyperparameters.w==0 
    error('trying to draw from improper prior');
end


[T,nvar]    = size(ydata);
[T2,nx]     = size(xdata);
% note that x must be same length as y, even though first part of x will not be used.
% This is so that the lags parameter can be changed without reshaping the xdata matrix.
if T2 ~= T, disp('Mismatch of x and y data lengths'),end 

T_prior=hyperparameters.w;

[YYdum, XXdum, ~,YYact,XXact] = get_dummyobs(ydata,xdata,lags,hyperparameters);

switch prior_only
    case 1 % prior only
        X = XXdum;
        y = YYdum;
    case -1 % flat prior
        X = XXact;
        y = YYact;
    otherwise % (proper) posterior
        X = [XXact; XXdum];
        y = [YYact; YYdum];
end
% OLS below
[vl,d,vr]   = svd(X(:,:),0);
di          = 1./diag(d);
B           = vl'*y;
B           = (vr.*repmat(di',nvar*lags+nx,1))*B;
u           = y-X(:,:)*B;
xxi         = vr.*repmat(di',nvar*lags+nx,1);
xxi         = xxi*xxi';
% B_alt=X\y;
B           = reshape(B,[nvar*lags+nx,nvar]); % rhs variables, equations


%-----------------------------------------------------------------
% Wishart and Gaussian draws below
omega   = zeros(nvar,nvar,ndraws); %covariance matrix
BB      = reshape(B,[(nvar*lags+nx)*nvar,1]); %vec(B)
Bnorm   = zeros((nvar*lags+nx)*nvar,ndraws); %reduced-form coefficients
By      = zeros(nvar,nvar,lags,ndraws); %stochastic coefficients
Bx      = zeros(nvar,nx,ndraws); %non-stochastic coefficients

biga = zeros(nvar*lags,nvar*lags);
for j = 1:lags-1
    biga(j*nvar+1:nvar*(j+1),nvar*(j-1)+1:j*nvar) = eye(nvar);
end

rho_max=1.03;
if prior_only==1
    T=0;
end
jj =0;
for n = 1 : ndraws
    if ndraws>100
        if mod(n,100)==0;fprintf('RF Draws: %d\n',n);end
    end
    %Z=zeros(size(u,1)-(nvar*lags+size(xdata,2)),nvar); %d.f. set to #obs.-#coeff.
    rho_actual=rho_max+1;
    while abs(rho_actual)>rho_max
        jj=jj+1;
       Z = zeros(T+T_prior*nvar,nvar); %corrected 1/11/03, degrees of freedom set to T = no. of obs.
       pos_definite = false;
       while ~pos_definite
           for i = 1 : size(Z,1)
              Z(i,:) = (chol(inv(u'*u))'*randn(nvar,1))';
           end
           omega(:,:,n) = inv(Z'*Z); %covariance matrix is drawn from inverse Wishart[inv(u'*u),#obs.-#coeff.]
           try
               Bnorm(:,n)   = BB+chol(kron(omega(:,:,n),xxi))'*randn((nvar*lags+nx)*nvar,1); %Bnorm are draws from Normal[Bhat,kron(omega,xxi)
               pos_definite = true;
           catch
               pos_definite = false;
           end           
       end
       b            = reshape(Bnorm(:,n),[nvar*lags+nx,nvar]);
       bb           = b(1:nvar*lags,:);
       bbb          = reshape(bb,nvar,lags,nvar);
       By(:,:,:,n)  = permute(bbb,[3,1,2]); %equations, variables, lags to match impulsdt.m
       for ii = 1:lags
            biga(1:nvar,(ii-1)*nvar+1:ii*nvar) = squeeze(By(:,:,ii,n));
       end
       rho_actual=max(abs(eig(biga)));
       if rho_actual>rho_max
           fprintf(2,sprintf('x%1.2f', rho_actual));
%        else
%           fprintf(2,'Y');
%           if jj>1000
%              fprintf(num2str(n,'::successful draw=%1.0f\\n'));
%           end
       end
%        if mod(jj,25)==0
%            fprintf(num2str(jj,'::%1.0f\\n'));
%        end
    end
   Bx(:,:,n)    = b(nvar*lags+(1:nx),:)';
end

